#lang plai-typed

(require (opaque-type-in racket/base [Syntax syntax?]))

(define-type ExprC
  [numC (n : number)
        (loc : Syntax)]
  [plusC (lhs : ExprC)
         (rhs : ExprC)
        (loc : Syntax)]
  [multC (lhs : ExprC)
         (rhs : ExprC)
        (loc : Syntax)]
  [argC (loc : Syntax)]
  [thisC (loc : Syntax)]
  [newC (class-name : symbol)
        (args : (listof ExprC))
        (loc : Syntax)]
  [getC (obj-expr : ExprC)
        (field-name : symbol)
        (loc : Syntax)]
  [sendC (obj-expr : ExprC)
         (method-name : symbol)
         (arg-expr : ExprC)
         (loc : Syntax)]
  [ssendC (obj-expr : ExprC)
          (class-name : symbol)
          (method-name : symbol)
          (arg-expr : ExprC)
          (loc : Syntax)])

(define-type ClassC
  [classC (name : symbol)
          (field-names : (listof symbol))
          (methods : (listof MethodC))])

(define-type MethodC
  [methodC (name : symbol)
           (body-expr : ExprC)])

;; ----------------------------------------

(define (make-find [name-of : ('a -> symbol)])
  (lambda ([name : symbol] [vals : (listof 'a)]) : 'a
    (cond
     [(empty? vals)
      (error 'find "not found")]
     [else (if (equal? name (name-of (first vals)))
               (first vals)
               ((make-find name-of) name (rest vals)))])))

(define find-class : (symbol (listof ClassC) -> ClassC)
  (make-find classC-name))
