#lang plai-typed
(require "class-loc.rkt"
         (opaque-type-in racket/base [Syntax syntax?]))

;; Tests of "inherit-loc.rkt" really belong here, too.

(define-type ExprI
  [numI (n : number)
        (loc : Syntax)]
  [plusI (lhs : ExprI)
         (rhs : ExprI)
         (loc : Syntax)]
  [multI (lhs : ExprI)
         (rhs : ExprI)
         (loc : Syntax)]
  [argI (loc : Syntax)]
  [thisI (loc : Syntax)]
  [newI (class-name : symbol)
        (args : (listof ExprI))
        (loc : Syntax)]
  [getI (obj-expr : ExprI)
        (field-name : symbol)
        (loc : Syntax)]
  [sendI (obj-expr : ExprI)
         (method-name : symbol)
         (arg-expr : ExprI)
         (loc : Syntax)]
  [superI (method-name : symbol)
          (arg-expr : ExprI)
          (loc : Syntax)])

(define-type ClassI
  [classI (name : symbol)
          (super-name : symbol)
          (field-names : (listof symbol))
          (methods : (listof MethodI))])

(define-type MethodI
  [methodI (name : symbol)
           (body-expr : ExprI)])

;; ----------------------------------------

(define (expr-i->c [a : ExprI] [super-name : symbol]) : ExprC
  (local [(define (recur expr)
            (expr-i->c expr super-name))]
    (type-case ExprI a
      [numI (n loc) (numC n loc)]
      [plusI (l r loc) (plusC (recur l) (recur r) loc)]
      [multI (l r loc) (multC (recur l) (recur r) loc)]
      [argI (loc) (argC loc)]
      [thisI (loc) (thisC loc)]
      [newI (class-name field-exprs loc)
            (newC class-name (map recur field-exprs) loc)]
      [getI (expr field-name loc)
            (getC (recur expr) field-name loc)]
      [sendI (expr method-name arg-expr loc)
             (sendC (recur expr)
                    method-name
                    (recur arg-expr)
                    loc)]
      [superI (method-name arg-expr loc)
              (ssendC (thisC loc)
                      super-name
                      method-name
                      (recur arg-expr)
                      loc)])))

(define (expr-loc [a : ExprI]) : Syntax
  (type-case ExprI a
    [numI (n loc) loc]
    [plusI (l r loc) loc]
    [multI (l r loc) loc]
    [argI (loc) loc]
    [thisI (loc) loc]
    [newI (class-name field-exprs loc) loc]
    [getI (expr field-name loc) loc]
    [sendI (expr method-name arg-expr loc) loc]
    [superI (method-name arg-expr loc) loc]))

;; ----------------------------------------

(define (method-i->c [m : MethodI] [super-name : symbol]) : MethodC
  (type-case MethodI m
    [methodI (name body-expr) 
             (methodC name 
                      (expr-i->c body-expr super-name))]))

;; ----------------------------------------

(define (class-i->c-not-flat [c : ClassI]) : ClassC
  (type-case ClassI c
    [classI (name super-name field-names methods)
            (classC
             name
             field-names
             (map (lambda (m) (method-i->c m super-name))
                  methods))]))

;; ----------------------------------------

(define (flatten-class [c : ClassC] 
                       [classes : (listof ClassC)] 
                       [i-classes : (listof ClassI)]) : ClassC
  (type-case ClassC c
    [classC (name field-names methods)
            (type-case ClassC (flatten-super name classes i-classes)
              [classC (super-name super-field-names super-methods)
                      (classC
                       name
                       (add-fields super-field-names field-names)
                       (add/replace-methods super-methods methods))])]))

(define (flatten-super [name : symbol]
                       [classes : (listof ClassC)] 
                       [i-classes : (listof ClassI)]) : ClassC
  (type-case ClassI (find-i-class name i-classes)
    [classI (name super-name field-names i-methods)
            (if (equal? super-name 'object)
                (classC 'object empty empty)
                (flatten-class (find-class super-name classes)
                               classes
                               i-classes))]))

;; ----------------------------------------

(define add-fields append)

(define (add/replace-methods [methods : (listof MethodC)]
                             [new-methods : (listof MethodC)])
  : (listof MethodC)
  (cond
    [(empty? new-methods) methods]
    [else (add/replace-methods
           (add/replace-method methods (first new-methods))
           (rest new-methods))]))

(define (add/replace-method [methods : (listof MethodC)] 
                            [new-method : MethodC])
  : (listof MethodC)
  (cond
    [(empty? methods) (list new-method)]
    [else
     (if (equal? (methodC-name (first methods))
                 (methodC-name new-method))
         (cons new-method (rest methods))
         (cons (first methods) 
               (add/replace-method (rest methods)
                                   new-method)))]))

;; ----------------------------------------

(define find-i-class : (symbol (listof ClassI) -> ClassI)
  (make-find classI-name))
