#lang racket/base
(require racket/list
         parser-tools/lex
         (prefix-in : parser-tools/lex-sre))

(provide lex
         content-tokens
         delimiter-tokens
         token->string)

(define-tokens content-tokens
  (ID NUMBER ERROR))

(define-empty-tokens delimiter-tokens
  (CLASS RETURN EXTENDS NUM
         THIS ARG SUPER NEW
         PLUS TIMES DOT COMMA
         COLON SEMICOLON
         OPENP CLOSEP
         OPENB CLOSEB
         WHITESPACE EOF))

(define lex
  (lexer-src-pos
   ["class" (token-CLASS)]
   ["extends" (token-EXTENDS)]
   ["return" (token-RETURN)]
   ["num" (token-NUM)]
   ["this" (token-THIS)]
   ["arg" (token-ARG)]
   ["super" (token-SUPER)]
   ["new" (token-NEW)]
   [":" (token-COLON)]
   [";" (token-SEMICOLON)]
   ["+" (token-PLUS)]
   ["*" (token-TIMES)]
   ["." (token-DOT)]
   ["," (token-COMMA)]
   ["(" (token-OPENP)]
   [")" (token-CLOSEP)]
   ["{" (token-OPENB)]
   ["}" (token-CLOSEB)]
   [(:: (:+ #\_ (:/ #\a #\z #\A #\Z))
        (:* #\_ (:/ #\a #\z #\A #\Z #\0 #\9)))
    (token-ID (string->symbol lexeme))]
   [(:+ (:/ #\0 #\9))
    (token-NUMBER (string->number lexeme))]
   [(:+ whitespace) (token-WHITESPACE)]
   [(eof) (token-EOF)]
   [any-char (token-ERROR lexeme)]))

;; Helper for error messages:
(define (token->string t v)
  (if v
      (format "~a" v)
      (format "~a" t)))
