#lang racket
(require (for-syntax syntax/parse
                     "typed-class-loc.rkt"
                     "inherit-loc.rkt"
                     "typed-parse-loc.rkt"
                     "inherit-parse-loc.rkt"
                     "unparse-class-loc.rkt")
         "method-macro.rkt"
         "class-macro.rkt"
         "typed-class-macro.rkt"
         "op-macro.rkt")

(provide this arg class new get send ssend
         (all-from-out "typed-class-macro.rkt")
         (rename-out [module-begin #%module-begin]
                     [datum #%datum]
                     [+-form +]
                     [*-form *]))

(define-syntax (module-begin stx)
  (syntax-parse stx
    #:literals (class)
    [(module-begin {class _ ...} ... expr)
     (syntax-parse #'expr
       #:literals (class)
       [{class _ ...}
        (raise-syntax-error 
         'module-begin
         "need an expression after classes")]
       [_
        ;; Parse
        (define t-classes
          (syntax-parse stx
            [(module-begin class-decl ... expr)
             (map parse-t-class
                  (syntax-e #'(class-decl ...)))]))
        (define i-a (parse #'expr))
        ;; Typecheck
        (typecheck i-a t-classes)
        ;; Convert to untyped (like `t-interp` plus `i-interp`)
        (define a (expr-i->c i-a 'object))
        (define i-classes (map strip-types t-classes))
        (define classes-not-flat (map class-i->c-not-flat i-classes))
        (define classes
          (map (lambda (c)
                 (flatten-class c classes-not-flat i-classes))
               classes-not-flat))
        ;; Unparse
        #`(#%module-begin
           #,@(map (lambda (c) (unparse-class c #'here)) classes)
           #,(unparse a #'here))])]))
    
(define-syntax (datum stx)
  (syntax-parse stx
    [(datum . v:number) #'(#%datum . v)]
    [(datum . other) (raise-syntax-error #f "bad syntax" #'other)]))
