#lang plai-typed
(require "class-loc.rkt")

(require (opaque-type-in racket/base [Syntax syntax?])
         (rename-in
          (typed-in racket/base
                    [datum->syntax : (Syntax s-expression Syntax -> Syntax)]
                    [values : (Syntax -> s-expression)])
          [values syntax->s-exp]))

(define (unparse [a : ExprC] [ctx : Syntax]) : Syntax
  (local [(define (to-stx s loc)
            (datum->syntax ctx s loc))
          (define (recur a)
            (syntax->s-exp (unparse a ctx)))]
    (type-case ExprC a
      [numC (n loc) (to-stx (number->s-exp n) loc)]
      [plusC (l r loc) (to-stx `{+ ,(recur l) ,(recur r)} loc)]
      [multC (l r loc) (to-stx `{* ,(recur l) ,(recur r)} loc)]
      [thisC (loc) (to-stx `this loc)]
      [argC (loc) (to-stx `arg loc)]
      [newC (class-name field-exprs loc)
            (to-stx `{new ,(symbol->s-exp class-name)
                          ,@(map recur field-exprs)}
                    loc)]
      [getC (obj-expr field-name loc)
            (to-stx `{get ,(recur obj-expr)
                          ,(symbol->s-exp field-name)}
                    loc)]
      [sendC (obj-expr method-name arg-expr loc)
             (to-stx `{send ,(recur obj-expr)
                            ,(symbol->s-exp method-name)
                            ,(recur arg-expr)}
                     loc)]
      [ssendC (obj-expr class-name method-name arg-expr loc)
              (to-stx `{ssend ,(recur obj-expr)
                              ,(symbol->s-exp class-name)
                              ,(symbol->s-exp method-name)
                              ,(recur arg-expr)}
                      loc)])))

(define (unparse-class [c : ClassC] [ctx : Syntax]) : Syntax
  (type-case ClassC c
    [classC (name field-names methods)
            (datum->syntax
             ctx
             `{class ,(symbol->s-exp name)
                {,@(map symbol->s-exp field-names)}
                ,@(map (lambda (m) (syntax->s-exp (unparse-method m ctx))) methods)}
             ctx)]))

(define (unparse-method [m : MethodC] [ctx : Syntax]) : Syntax
  (type-case MethodC m
    [methodC (name expr)
             (datum->syntax
              ctx
              `{,(symbol->s-exp name) ,(syntax->s-exp
                                        (unparse expr ctx))}
              ctx)]))
